==> camitk_3.3.2.orig.tar.gz: camitk-3.3.2-Source/sdk/components/msh/MshExtension.cpp <==
/*****************************************************************************
 * $CAMITK_LICENCE_BEGIN$
 *
 * CamiTK - Computer Assisted Medical Intervention ToolKit
 * (c) 2001-2014 UJF-Grenoble 1, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
 *
 * Visit http://camitk.imag.fr for more information
 *
 * This file is part of CamiTK.
 *
 * CamiTK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * CamiTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with CamiTK.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $CAMITK_LICENCE_END$
 ****************************************************************************/

#include "MshComponent.h"
#include "MshExtension.h"

//-- Qt
#include <QFileInfo>
#include <QMessageBox>

//-- VTK
#include <vtkCellType.h>
#include <vtkCell.h>

//-- CamiTK
#include <Log.h>
#include <Application.h>
using namespace camitk;

// declare the plugin
Q_EXPORT_PLUGIN2(MshExtension, MshExtension)


QString MshExtension::getName() const {
    return "Msh Component";
}

QString MshExtension::getDescription() const {
    return "This simple MSH Component allows you to manipulate <em>.msh</em> files (initially from gmsh software). See <a href=\"http://www.geuz.org/gmsh/doc/texinfo/gmsh.html\">http://www.geuz.org/gmsh/doc/texinfo/gmsh.html</a> for more information about this format";
}

QStringList MshExtension::getFileExtensions() const {
    QStringList ext;
    ext << "msh";
    return ext;
}

Component * MshExtension::open(const QString & fileName) throw (AbortException) {
    return new MshComponent(fileName);
}

bool MshExtension::save(Component* component) const {
    bool saveOk = true;

    vtkSmartPointer<vtkPointSet> ps =  component->getPointSet();

    if (ps == NULL || ps->GetNumberOfPoints()==0 || ps->GetNumberOfCells() == 0) {
        QMessageBox::warning(NULL, "Save As msh", "The selected component does not have any points or cells. This is an invalid mesh.", QMessageBox::Abort);
        return false;
    }

    QString baseFilename = QFileInfo(component->getFileName()).absolutePath() + "/" + QFileInfo(component->getFileName()).completeBaseName();

    QString mshFilename = baseFilename + ".msh";

    ofstream mshFile ( mshFilename.toUtf8() );
    double xyz[3];
    mshFile << "$NOD" << endl;
    mshFile << ps->GetPoints()->GetNumberOfPoints() << endl;
    for ( int i = 0; i < ps->GetPoints()->GetNumberOfPoints(); i++ ) {
        ps->GetPoints()->GetPoint(i, xyz);
        // write atom
        mshFile << i + 1 << " " << xyz[0] << " " << xyz[1] << " " << xyz[2] << endl;
    }
    mshFile << "$ENDNOD" << endl;
    mshFile << "$ELM" << endl;
    mshFile << ps->GetNumberOfCells() << endl;

    for ( vtkIdType i = 0; i < ps->GetNumberOfCells() ; i++ ) {
        switch (ps->GetCell(i)->GetCellType()) {
        case VTK_LINE :
            mshFile << i + 1 << " 1 1 1 2 ";
            for (vtkIdType j=0; j<ps->GetCell(i)->GetNumberOfPoints(); j++)
                mshFile << ps->GetCell(i)->GetPointId(j) + 1 << " ";
            mshFile << endl;
            break;
        case VTK_TETRA :
            mshFile << i + 1 << " 4 1 1 4 "
                    << ps->GetCell(i)->GetPointId(0) + 1 << " "
                    << ps->GetCell(i)->GetPointId(2) + 1 << " "
                    << ps->GetCell(i)->GetPointId(1) + 1 << " "
                    << ps->GetCell(i)->GetPointId(3) + 1 << endl;
            break;
        case VTK_HEXAHEDRON :
            mshFile << i + 1 << " 5 1 1 8 ";
            for (vtkIdType j=0; j<ps->GetCell(i)->GetNumberOfPoints(); j++)
                mshFile << ps->GetCell(i)->GetPointId(j) + 1 << " ";
            mshFile << endl;
            break;
        case VTK_WEDGE :
            mshFile << i + 1 << " 6 1 1 6 ";
            for (vtkIdType j=0; j<ps->GetCell(i)->GetNumberOfPoints(); j++)
                mshFile << ps->GetCell(i)->GetPointId(j) + 1 << " ";
            mshFile << endl;
            break;
        case VTK_PYRAMID :
            mshFile << i + 1 << " 7 1 1 5 ";
            for (vtkIdType j=0; j<ps->GetCell(i)->GetNumberOfPoints(); j++)
                mshFile << ps->GetCell(i)->GetPointId(j) + 1 << " ";
            mshFile << endl;
            break;
        case VTK_TRIANGLE :
            mshFile << i + 1 << " 2 1 1 3 ";
            for (vtkIdType j=0; j<ps->GetCell(i)->GetNumberOfPoints(); j++)
                mshFile << ps->GetCell(i)->GetPointId(j) + 1 << " ";
            mshFile << endl;
            break;
        case VTK_QUAD :
            mshFile << i + 1 << " 3 1 1 4 ";
            for (vtkIdType j=0; j<ps->GetCell(i)->GetNumberOfPoints(); j++)
                mshFile << ps->GetCell(i)->GetPointId(j) + 1 << " ";
            mshFile << endl;
            break;
        default:
            saveOk = false;
            CAMITK_ERROR("VolumetricMeshExtension", "save", "unsupported element");
            break;
        }
    }

    mshFile << "$ENDELM" << endl;
    mshFile.close();

    return saveOk;
}

==> camitk_3.4.0.orig.tar.gz: camitk-3.4.0-Source/sdk/components/msh/MshExtension.cpp <==
/*****************************************************************************
 * $CAMITK_LICENCE_BEGIN$
 *
 * CamiTK - Computer Assisted Medical Intervention ToolKit
 * (c) 2001-2014 UJF-Grenoble 1, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
 *
 * Visit http://camitk.imag.fr for more information
 *
 * This file is part of CamiTK.
 *
 * CamiTK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * CamiTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with CamiTK.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $CAMITK_LICENCE_END$
 ****************************************************************************/

#include "MshComponent.h"
#include "MshExtension.h"

//-- Qt
#include <QFileInfo>
#include <QMessageBox>

//-- VTK
#include <vtkCellType.h>
#include <vtkCell.h>

//-- CamiTK
#include <Log.h>
#include <Application.h>
using namespace camitk;

// declare the plugin
Q_EXPORT_PLUGIN2 ( MshExtension, MshExtension )


QString MshExtension::getName() const {
    return "Msh Component";
}

QString MshExtension::getDescription() const {
    return tr("This simple MSH Component allows you to manipulate <em>.msh</em> files (initially from gmsh software). See <a href=\"http://www.geuz.org/gmsh/doc/texinfo/gmsh.html\">http://www.geuz.org/gmsh/doc/texinfo/gmsh.html</a> for more information about this format");
}

QStringList MshExtension::getFileExtensions() const {
    QStringList ext;
    ext << "msh";
    return ext;
}

Component * MshExtension::open ( const QString & fileName ) throw ( AbortException ) {
    return new MshComponent ( fileName );
}

bool MshExtension::save ( Component* component ) const {
    bool saveOk = true;
    bool found = true;

    vtkSmartPointer<vtkPointSet> ps =  component->getPointSet();

    if ( ps == NULL || ps->GetNumberOfPoints() ==0 || ps->GetNumberOfCells() == 0 ) {
        QMessageBox::warning ( NULL, "Save As msh", tr("The selected component does not have any points or cells. This is an invalid mesh."), QMessageBox::Abort );
        return false;
    }

    QString baseFilename = QFileInfo ( component->getFileName() ).absolutePath() + "/" + QFileInfo ( component->getFileName() ).completeBaseName();

    QString mshFilename = baseFilename + ".msh";

    ofstream mshFile ( mshFilename.toUtf8() );
    double xyz[3];
    mshFile << "$NOD" << endl;
    mshFile << ps->GetPoints()->GetNumberOfPoints() << endl;
    for ( int i = 0; i < ps->GetPoints()->GetNumberOfPoints(); i++ ) {
        ps->GetPoints()->GetPoint ( i, xyz );
        mshFile << i + 1 << " " << xyz[0] << " " << xyz[1] << " " << xyz[2] << endl;
    }
    mshFile << "$ENDNOD" << endl;
    mshFile << "$ELM" << endl;
    mshFile << ps->GetNumberOfCells() << endl;

    for ( vtkIdType i = 0; i < ps->GetNumberOfCells() ; i++ ) {
        found = true;
        switch ( ps->GetCell ( i )->GetCellType() ) {
        case VTK_LINE :
            mshFile << i + 1 << " 1 1 1 2";
            break;
        case VTK_TETRA :
            mshFile << i + 1 << " 4 1 1 4";
            break;
        case VTK_HEXAHEDRON :
            mshFile << i + 1 << " 5 1 1 8";
            break;
        case VTK_WEDGE :
            mshFile << i + 1 << " 6 1 1 6";
            break;
        case VTK_PYRAMID :
            mshFile << i + 1 << " 7 1 1 5";
            break;
        case VTK_TRIANGLE :
            mshFile << i + 1 << " 2 1 1 3";
            break;
        case VTK_QUAD :
            mshFile << i + 1 << " 3 1 1 4";
            break;
        default:
            saveOk = false;
            found = false;
            CAMITK_ERROR ( "VolumetricMeshExtension", "save", "unsupported element" );
            break;
        }
        if ( found ) {
            for ( vtkIdType j=0; j<ps->GetCell ( i )->GetNumberOfPoints(); j++ )
                mshFile << " " << ps->GetCell ( i )->GetPointId ( j ) + 1;
            mshFile << endl;
        }
    }

    mshFile << "$ENDELM" << endl;
    mshFile.close();

    return saveOk;
}
